/* 
 *
 * GULP & SASS Starter Toolkit
 * by David Rosseljong
 * https://david-rosseljong.com
 * 
 * Required gulp plugins for the project.
 * Comment out or delete what you don't need.
 * Better yet, npm uninstall them BUT be sure
 * about what you are doing.
 *
 */
const gulp = require('gulp');
const sass = require('gulp-sass');
const sourcemaps = require('gulp-sourcemaps');
const browserSync = require('browser-sync').create();
const pug = require('gulp-pug');
const image = require('gulp-image');
const prefix = require('gulp-autoprefixer');
const newer = require('gulp-newer');
const wait = require('gulp-wait');

/* 
 *
 * The SASS task. 
 * It creates our compressed & prefixed css 
 * file including sourcemaps for the browser.
 * 
 * Wait is included to give it time so that it
 * can find all files main.sass needs. If you have
 * a lot of files, occasionally it throws an error
 * that it didn't found an importet sass file.
 *
 */
gulp.task('sass', () => {
  return gulp.src('src/sass/main.sass')
    .pipe(wait(50))
    .pipe(sourcemaps.init())
    .pipe(sass({
      errLogToConsole: true,
      outputStyle: 'compressed'
    }).on('error', sass.logError))
    .pipe(prefix('last 5 versions'))
    .pipe(sourcemaps.write('/maps'))
    .pipe(gulp.dest('build/css'))
    .pipe(browserSync.stream());
});

/* 
 *
 * The IMAGE task.
 * Checks if images are new in the src folder,
 * then they get compressed and copied over
 * to the build folder if they are, well, new.
 *
 */
gulp.task('image', () => {
  return gulp.src('src/img/**/*')
    .pipe(newer('build/img'))
    .pipe(image())
    .pipe(gulp.dest('build/img'));
});

/* 
 *
 * The PUG task.
 * Prettifies our pug files after compiling.
 * All files in src/pug are compiled to html,
 * so that you can have index.html, about.html etc.
 * Sub-directories are not compiled. They are for
 * including.
 * 
 * Set pretty to false to minify html.
 * 
 * Please check #5 on https://gulpsasstoolkit.com/docs.html
 * for the code if you want the gulp task as it is optional.
 *
 */

/* Delete this comment or replace with the pug task. */

/* 
 *
 * The COPY task
 * Copy files from node_modules or src folder
 * to the build folder. E.g. Bootstrap, jQuery.
 * 
 * Please check #4 on https://gulpsasstoolkit.com/docs.html
 * for the code if you want the copy task as it is optional.
 *
 */

/* Delete this comment or replace with the copy task. */

/* 
 *
 * The SERVE task.
 * Initializing BrowserSync and specifying our build folder.
 *
 */
gulp.task('serve', ['sass'], () => {
  browserSync.init({
    server: {
      baseDir: './build'
    }
  });
});

/* 
 *
 * The WATCH task.
 * Watch all our files for changes, then run tasks above accordingly.
 *
 */
gulp.task('watch', ['serve'], () => {
  gulp.watch('src/sass/**/*.sass', ['sass'])
  gulp.watch('src/pug/**/**/*.pug', ['pug'])
  gulp.watch('src/img/**/*', ['image'])
});

/* 
 *
 * The DEFAULT task.
 * This is our default task.
 * Nothing fancy about it. :)
 *
 */
gulp.task('default', ['watch', 'image']);

/* 
 *
 * Task are running in this order:
 * 
 * 1. DEFAULT task starts WATCH & IMAGE tasks.
 * 2. WATCH task only runs if SERVE task is done.
 * 3. SERVE task only runs if SASS & PUG tasks are done.
 *
 * If you need a more in depth explanation on this, please ask.
 * 
 */